
/***********************************************************************
sw_disp.c

		peak detector gui/for sweden flt  7Feb06/mm
		adapted from pd_disp.c

rev 1.0:	 7Feb06--first version
rev 2.0		11Mar06--updated to CVI Labwindows
rev 2.1		13Mar06--added light curve display
rev 2.3		30Mar06--added pps and tilt displays

************************************************************************

This program maintains a display of the returned peak detector data.

***********************************************************************/
#include <utility.h>
#include <rs232.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include "sw.h"
#define FIFTYms (CLOCKS_PER_SEC * 0.050)
#define PORT 0
#define DFILE 1

typedef unsigned char BYTE;
typedef unsigned short WORD;

typedef struct {
    short width[208];
    short x[208];
    long y[208];
    int reps[4];
    float cal;
} spec;

int     service(int, int, spec *);
int     get_frm(BYTE *, int);
int     resync(int);
void    init(spec *);
void    unpack(BYTE *, int *, int);
void    frm_process(BYTE *, spec *, int, int);

static FILE *inf, *ouf;
static long garbage=0;

	        
/**********************************************************************
			Start of main function.
**********************************************************************/
main(int argc, char *argv[])
{
    int     quit,
	    src;
    int     mode,
	    panel,
	    status,
	    menu;
    static spec spectrum;
    
    if (argc == 1) {
	src = PORT;
	quit=OpenComConfig(1,"COM1", 19200, 0, 8, 1, 0, 0);
	ouf = fopen("swgse.dat","wb");
	if (quit < 0) {
	    printf("%d COM1 port error %d\n", quit);
	    return 0;
	}
    }

    else if (argc == 2) {
	if ((inf = fopen(argv[1], "rb")) == NULL) {
	    printf("Cannot open input file %s\n", argv[1]);
	    return 0;
	}
	src=DFILE;
    }
    quit = 0;
    init(&spectrum);
    panel = LoadPanel(0,"sw.uir", SW);
    status = DisplayPanel(panel);
    while (!quit) {
	quit = service(panel, src, &spectrum);
    }
    DiscardPanel(panel); 
    if (src==PORT)
	CloseCom(1);
    else
    	fclose(inf);
    fclose(ouf);
    return 0;
}  /* end of main function */



/**********************************************************************
    This function services the display
**********************************************************************/
int
service(int panel, int src, spec *p)
{
    static int i,
	    j,
	    quit,
	    mode,
	    status,
	    isevent,
	    con,
	    pan;
    static BYTE hex[521];
    static int xmin,
	    xmax,
	    offset;
    static int temp,
	    byte_cnt;
    static long acq_cntr, cntr;
    static float ymin,ymax,diffspec[208];
    static BYTE inbuf[256];
    static char msg[30];

    quit = 0;

/*
	This positions us at the end of the last incoming serial block
	For 19.2k baud and 256 byte frames, data comes in 133ms bursts
	followed by 867ms of quiet.
	First require 100ms period of quiet, which means it is now
	after the last record has come in. Then extend the timeout
	to 1+ seconds.
*/
    if (src == PORT) {
	SetCtrlVal(panel, SW_STATUS, "setting up serial port"); 
	SetComTime(1, 0.1);
	while (ComRdByte(1) >= 0);
	SetComTime(1, 1.05);
	acq_cntr=1000;
	SetCtrlVal(panel, SW_STATUS, "serial port ready");  
    }
    else
	acq_cntr=10000;
    xmin = 0;
    xmax = 4095;
    ymin = 1.e-3;
    ymax = 10.0;
    mode = 1;
    for (i = 0; i < 208; i++)
	p->y[i] = 0.0;
    while (!quit) {
	if ((isevent = GetUserEvent(0, &pan, &con))) {
	    switch (con) {
	    case SW_ACQ:
		GetCtrlVal(pan, con, &mode);
		if (!mode)
		    SetCtrlAttribute(pan, SW_CNT, 10, 2);
		break;
	    case SW_CNT:
		GetCtrlVal(pan, con, &acq_cntr);
		break;
	    case SW_CLR:
		for (i = 0; i < 208; i++)
		    p->y[i] = 0;
		p->reps[0]=p->reps[1]=p->reps[2]=p->reps[3]=0;
		DeleteGraphPlot(panel, SW_SPEC, -1, 1);
		break;
	    case SW_XLOGLIN:
		GetCtrlVal(pan, con, &temp);
		SetCtrlAttribute(pan, SW_SPEC, ATTR_XMAP_MODE,
		  temp?VAL_LOG:VAL_LINEAR);
		break;
	    case SW_YLOGLIN:
		GetCtrlVal(pan, con, &temp);
		if (temp==VAL_LOG && ymin<1e-4) {
		    ymin=1e-4;
		    SetAxisRange(pan, SW_SPEC, VAL_NO_CHANGE, xmin, xmax,
		      VAL_MANUAL, ymin, ymax);
		}
		SetCtrlAttribute(pan, SW_SPEC, ATTR_YMAP_MODE,
		  temp?VAL_LOG:VAL_LINEAR);
		break;
	    case SW_XMIN:
		GetCtrlVal(pan, con, &xmin);
		SetAxisRange(pan, SW_SPEC, VAL_MANUAL, xmin, xmax,
		  VAL_NO_CHANGE, ymin, ymax);
		break;
	    case SW_XMAX:
		GetCtrlVal(pan, con, &xmax);
		SetAxisRange(pan, SW_SPEC, VAL_MANUAL, xmin, xmax,
		  VAL_NO_CHANGE, ymin, ymax);
		break;
	    case SW_YMIN:
		GetCtrlVal(pan, con, &ymin);
		SetAxisRange(pan, SW_SPEC, VAL_NO_CHANGE, xmin, xmax,
		  VAL_MANUAL, ymin, ymax);
		break;
	    case SW_YMAX:
		GetCtrlVal(pan, con, &ymax);
		SetAxisRange(pan, SW_SPEC, VAL_NO_CHANGE, xmin, xmax,
		  VAL_MANUAL, ymin, ymax);
		break;
	    case SW_QUIT:
		return(1);
		break;
	    default:
		return 0;
	    }
	    continue;
	}
	if (get_frm(inbuf, src)) {
	    if (src == PORT)
		sprintf(msg, "COM1 err: too few bytes read");
	    else						  
		sprintf(msg, "EOF on input file");
	}
	else {
	    msg[0] = ' ';
	    msg[1] = '\0';
	    if (src==PORT)
	    	fwrite(inbuf,256,1,ouf);
	    frm_process(inbuf, p, panel, mode);
	    if (mode)
		if (acq_cntr != 0) {
		    DeleteGraphPlot(panel, SW_SPEC, -1, 1);
		    for (i=0; i<208; i++) {
		    	cntr=p->reps[i/52];
		    	if (cntr)
		    	    diffspec[i] = (float) p->y[i]/p->width[i]/cntr/p->cal;
		    	else
		    	    diffspec[i]=0;
		    }
		    PlotXY(panel, SW_SPEC, p->x, diffspec, 208, VAL_SHORT_INTEGER,
		      VAL_FLOAT, VAL_THIN_STEP, 0, VAL_SOLID,1, VAL_BLUE);
		    acq_cntr--;
		    SetCtrlVal(panel, SW_CNT, acq_cntr);
		} else {
		    SetCtrlAttribute(panel, SW_CNT, 10, 2);
		    mode = 0;
		    SetCtrlVal(panel, SW_ACQ, 0);
		}
	    for (i = 0, j = 0; i < 256; i++)
		if (i % 32 == 0) {
		    sprintf(hex + j, "\n%02X", inbuf[i]);
		    j += 3;
		} else {
		    sprintf(hex + j, "%02X", inbuf[i]);
		    j += 2;
		}
	    hex[520] = '\0';
	    SetCtrlVal(panel, SW_RAW, hex);
	    SetCtrlVal(panel, SW_JUNK, garbage);
	}
	SetCtrlVal(panel, SW_STATUS, msg);
    }
    return 0;
} /******* END service() ********/


/* get_frm acquires a fr ame of data from one of several srcs
	INPUT p-> 256 byte buffer into which a frame is written
	      src is a code to indicate where the data comes from
	      		DFILE: means a tm file
			PORT: means COM1 port at 19.2k, no header
	OUTPUT	0 if data was retrieved
		-1 if timeout on serial port or EOF for file data

	buffer will be filled with a frame if successful
	if data is coming from a file, we will also wait for 100ms
	corresponding to a readback 10X faster than real time
*/
int
get_frm(BYTE *p, int src)
{
    static int byte_cnt;
    static unsigned short x, i, temp, chksum;

    switch (src) {
    case PORT:
	if ((byte_cnt = ComRd(1, p, 256)) < 256)
	    return -1;
	return 0;
    case DFILE:
	Delay(0.2);	   
	while (fread(p, 256, 1, inf) == 1) {
	    chksum = (*(p+254)<<8) | *(p+255);
	    x=0;
	    i=0;
	    while (i<254) {
		temp = *(p+(i++))<<8;
		temp |= *(p+(i++));
		x += temp;
	    }
	    if (x==chksum)
		return 0;
	    if (resync(src))
	    	break;
	}
	return -1;
    default:
	return -1;
    }
}

void
init(spec *p)
{
    static int i;

    for (i = 0; i < 30; i++)
	p->width[i] = 2;
    for (i = 30; i < 77; i++)
	p->width[i] = 4;
    for (i = 77; i < 107; i++)
	p->width[i] = 8;
    for (i = 107; i < 136; i++)
	p->width[i] = 16;
    for (i = 136; i < 183; i++)
	p->width[i] = 32;
    for (i = 183; i < 208; i++)
	p->width[i] = 64;
    
    p->x[0] = 0;
    for (i = 1; i < 208; i++)
	p->x[i] = p->x[i - 1] + p->width[i];
    for (i = 0; i < 208; i++)
	p->x[i] += p->width[i] / 2;
    p->cal = 2.4;
    return;
}

void
unpack(BYTE * p, int *q, int cnt)
{
    static unsigned short temp;

    while (cnt > 0) {
	temp = (*p++) << 4;
	temp |= ((*p) >> 4);
	*q++ = temp;
	temp = ((*p++) << 8) & 0xFFF;
	temp |= *p++;
	*q++ = temp;
	cnt -= 2;
    }
    return;
}

void
frm_process(BYTE *p, spec *sp, int pan, int mode)
{
    static char timestr[11];
    static int i,
	    j,
	    q[52],
	    temp,
	    hh,
	    mm;
    static BYTE fctype;
    static WORD lc[80];
    static short stemp;
    static long ltemp;
    static float ftemp;
    static double dtemp;

    fctype = *(p + 5) & 0x3;
    temp = (*(p + 244) << 8) | *(p + 245);
    SetCtrlVal(pan, SW_IRQ, temp);
    temp = (*(p + 10) << 8) | *(p + 11);
    SetCtrlVal(pan, SW_LL, temp);
    temp = (*(p + 12) << 8) | *(p + 13);
    SetCtrlVal(pan, SW_PD, temp);
    temp = (*(p + 14) << 8) | *(p + 15);
    SetCtrlVal(pan, SW_HL, temp);
    if (fctype ==0 || fctype==2) {
	stemp = (*(p + 246) << 8) | *(p + 247);
	ftemp = stemp / 1.0e4;
	SetCtrlVal(pan, SW_MAGX, ftemp);
	stemp = (*(p + 248) << 8) | *(p + 249);
	ftemp = stemp / 1.0e4;
	SetCtrlVal(pan, SW_MAGY, ftemp);
	stemp = (*(p + 250) << 8) | *(p + 251);
	ftemp = stemp / 1.0e4;
	SetCtrlVal(pan, SW_MAGZ, ftemp);
    } else {
	stemp = (*(p + 246) << 8) | *(p + 247);
	ftemp = stemp/10.0;
	SetCtrlVal(pan, SW_PITCH, ftemp);
	stemp = (*(p + 248) << 8) | *(p + 249);
	ftemp = stemp/10.0;
	SetCtrlVal(pan, SW_ROLL, ftemp);
    }
    temp = (*(p + 252) << 8) | *(p + 253);
    SetCtrlVal(pan, SW_PPS, temp);

    if (mode) {
	unpack(p + 166, q, 52);
	temp = 52 * fctype;
	sp->reps[fctype]++;
	if (mode>0)
	    for (i = 0; i < 52; i++)
		sp->y[i + temp] += q[i];
	else
	    for (i = 0; i < 52; i++)
		sp->y[i + temp] -= q[i];
    }
    unpack(p+16,q,20);
    PlotStripChart(pan,SW_PHOT,q,20,0,0,VAL_INTEGER);
    
    j=0;
    for (i=46; i<166; i+=6) {
    	lc[j++]=*(p+i)<<8 | *(p+i+1);
    	lc[j++]=*(p+i+2)<<8 | *(p+i+3);
    	lc[j++]=*(p+i+4);
    	lc[j++]=*(p+i+5);
    }
    PlotStripChart(pan,SW_LC,lc,80,0,0,VAL_UNSIGNED_SHORT_INTEGER);

    ltemp = (*(p + 6) << 8) | *(p + 7);
    ltemp = (ltemp << 8) | *(p + 8);
    ltemp = (ltemp << 8) | *(p + 9);
    switch (fctype) {
    case 0:	/* Altitude */
	dtemp = ltemp * 1.e-3;
	SetCtrlVal(pan, SW_ALT, dtemp);
	break;
    case 1:	/* Time */
	dtemp = ltemp * 1.e-3;
	dtemp -= 8.64e4 * ((int) (dtemp / 8.64e4));
	hh = dtemp / 3.6e3;
	dtemp -= 3.6e3 * hh;
	mm = dtemp / 6.e1;
	dtemp -= 6.0e1 * mm;
	sprintf(timestr, "%02d:%02d:%04.1f", hh, mm, dtemp);
	SetCtrlVal(pan, SW_TIME, timestr);
	break;
    case 2:	/* Latitude */
	dtemp = ltemp * 8.38190e-8;
	SetCtrlVal(pan, SW_LAT, dtemp);
	break;
    case 3:	/* Longitude */
	dtemp = ltemp * 8.38190e-8;
	SetCtrlVal(pan, SW_LON, dtemp);
    default:
	break;
    }
    return;
}
/* Back up 255 bytes in input file and search for sync word
	returns -1 on failure; 0 on success
*/
int
resync(int s)
{
    static unsigned short temp, state;
    static BYTE x;

    state = 0;
    fseek(inf,-255L,SEEK_CUR);
    garbage++;
    while ((temp=getc(inf))!=EOF) {
	garbage++;
	x = temp;
	switch (state) {
	case 0:
	    if (x==0xEB)
		state=1;
	    break;
	case 1:
	    if (x==0x90) {
	    	fseek(inf,-2L,SEEK_CUR);
		garbage -= 2;
		return 0;
	    }
	    state = 0;
	    break;
	default:
	    state=0;
	    break;
	}
    }
    return 1;
}
